 PROGRAM
   MAP
     CheckOpen(FILE)
     DiskError(<STRING>),BYTE
     INIRestoreWindow(STRING)
     INISaveWindow(STRING)
     MODULE('TOUR001.clw')
       Main 
     END
     MODULE('TOUR002.clw')
       ListByCompany 
     END
     MODULE('TOUR003.clw')
       UpdateCompany 
     END
     MODULE('TOUR004.clw')
       LookUpState 
     END
   END
   INCLUDE('Equates.CLW')
   INCLUDE('Keycodes.CLW')
   INCLUDE('Errors.CLW')


Action        LONG,THREAD

Names                  FILE,DRIVER('CLARION'),CREATE,PRE(NAM)
CompanyNameKey           KEY(NAM:CompanyName,NAM:LastName,NAM:FirstName),DUP,NOCASE,OPT
NameKey                  KEY(NAM:LastName,NAM:FirstName),DUP,NOCASE,OPT
ContactNoKey             KEY(NAM:ContactNo),NOCASE,OPT
Record                   RECORD
CompanyName                 STRING(35)
FirstName                   STRING(15)
LastName                    STRING(20)
Address                     STRING(35)
City                        STRING(20)
State                       STRING(2)
ZipCode                     STRING(10)
ContactNo                   LONG
                         END
                       END

Names::Used LONG

States                 FILE,DRIVER('CLARION'),CREATE,PRE(STA)
StateCodeKey             KEY(STA:StateCode),NOCASE,OPT
StateNameKey             KEY(STA:StateName),NOCASE,OPT
Record                   RECORD
StateCode                   STRING(2)
StateName                   STRING(20)
                         END
                       END

States::Used LONG

   CODE
   Main

CheckOpen         PROCEDURE(File)

  CODE
  OPEN(File,22h)                                  !Attempt to open the file
  CASE ERRORCODE()                                ! and check for errors
  OF NoError                                      !Return if no error
  OROF IsOpenErr                                  ! or if already open.
    DO ProcedureReturn
  OF NoFileErr                                    !If file was not found
    CREATE(File)                                  !Create the file
    IF ERRORCODE()
      IF MESSAGE('Failure (' & ERROR() & ') creating ' & CLIP(NAME(File)) & '.  Press OK to return to DOS.','File Creation Error',ICON:Exclamation,Button:OK,BUTTON:OK,1)
        HALT('File Creation Error!')
      END
    END
    OPEN(File,22h)                                ! then open it
    IF ~ERRORCODE()                               !  And return if it opened
      DO ProcedureReturn
    ELSE
      HALT(ERRORCODE())
    END
  OF InvalidFileErr                               !Invalid Record Declaration
    IF MESSAGE('Error (' & ERROR() & ') accessing ' & CLIP(NAME(File)) & '.  Press OK to return to DOS.','Invalid File',ICON:Exclamation,Button:OK,BUTTON:OK,1)
      HALT('Invalid File!')
    END
  OF BadKeyErr                                    !Key Files must be rebuilt
    IF MESSAGE(CLIP(NAME(File)) & ' key file is invalid.  Do you want to rebuild the key?','Invalid Key',ICON:Question,Button:Yes + Button:No,BUTTON:Yes,1) = Button:Yes
      OPEN(File,12H)                               !Open for exclusive access
      BUILD(File)                                  !Rebuild the key files
    ELSE
      HALT('Invalid Key!')
    END
    IF ERRORCODE()
      IF MESSAGE('Error (' & ERROR() & ') repairing key for ' & CLIP(NAME(File)) & '.  Press OK to return to DOS.','Key Rebuild Error',ICON:Exclamation,Button:OK,BUTTON:OK,1)
        HALT('Error Rebuilding Key!')
      END
    ELSE
      CLOSE(File)                                 !Close
      OPEN(File,22h)                              ! then open it
    END
  END                                             !End of Case Structure
  IF DiskError(NAME(File) & ' File could not be opened') THEN HALT(0). !Cannot resume
  DO ProcedureReturn
ProcedureReturn ROUTINE
  RETURN

DiskError        FUNCTION(Cause)
StopMsg::        STRING(180)
LOC::ReturnVal   BYTE(0)

  CODE
  IF ~ERRORCODE() THEN DO ProcedureReturn.        !Return with no error
  IF ~OMITTED(1)                                  !If a cause was given
    StopMsg:: = 'Cause: ' & Cause                 !  Display it
  END                                             !End IF
  IF ERRORFILE()                                  !If error involves a file
    StopMsg:: = CLIP(StopMsg::) & ' File : ' |    !  display the file
              & ERRORFILE()
  END                                             !End IF
  StopMsg:: = CLIP(StopMsg::) & ' Error: '   |    !Display the error code
            & ERRORCODE() & ' - '            |    !  and the error message
            & ERROR()

  IF MESSAGE(CLIP(StopMsg::),'Disk Error',ICON:Exclamation,Button:OK,Button:OK,1) !Stop with message
    LOC::ReturnVal = True                         !Return with error
  END
  DO ProcedureReturn
ProcedureReturn ROUTINE
  RETURN(LOC::ReturnVal)

INIRestoreWindow   PROCEDURE(ProcedureName)

INI::XPosit        LONG,AUTO
INI::YPosit        LONG,AUTO
INI::Width         LONG,AUTO
INI::Height        LONG,AUTO
INI::String        STRING(20),AUTO
INI::FileName      CSTRING('TOUR.INI')

  CODE
  LOOP
    IF TARGET{Prop:Maximize} = FALSE
      INI::XPosit = GETINI(ProcedureName,'XPosit',,INI::FileName)
      IF NOT INI::XPosit THEN BREAK.
      INI::YPosit = GETINI(ProcedureName,'YPosit',,INI::FileName)
      IF NOT INI::YPosit THEN BREAK.
      IF TARGET{Prop:Resize}
        INI::Width = GETINI(ProcedureName,'Width',,INI::FileName)
        IF NOT INI::Width THEN BREAK.
        INI::Height = GETINI(ProcedureName,'Height',,INI::FileName)
        IF NOT INI::Height THEN BREAK.
      ELSE
        INI::Width = TARGET{Prop:Width}
        INI::Height = TARGET{Prop:Height}
      END
      SETPOSITION(0,INI::XPosit,INI::YPosit,INI::Width,INI::Height)
    END
    BREAK
  END
  INI::String = GETINI(ProcedureName,'Maximize',,INI::FileName)
  IF INI::String = 'Yes'
    TARGET{Prop:Maximize}=1
  ELSE
    TARGET{Prop:Maximize}=0
  END
  INI::String = GETINI(ProcedureName,'Iconize',,INI::FileName)
  IF INI::String = 'Yes'
    TARGET{Prop:Iconize}=1
  ELSE
    TARGET{Prop:Iconize}=0
  END
  RETURN

INISaveWindow   PROCEDURE(ProcedureName)
INI::Section       CSTRING(80)
INI::String        CSTRING(20)
INI::Entry         CSTRING(20)
INI::FileName      CSTRING('TOUR.INI')
  CODE
  INI::Section = ProcedureName
  INI::Entry = 'Iconize'
  IF Target{Prop:Iconize}
    INI::String = 'Yes'
  ELSE
    INI::String = 'No'
  END
  TARGET{PROP:Iconize} = 0
  PUTINI(INI::Section,INI::Entry,INI::String,INI::FileName)
  INI::Entry = 'Maximize'
  IF Target{Prop:Maximize}
    INI::String = 'Yes'
  ELSE
    INI::String = 'No'
  END
  TARGET{PROP:Maximize} = 0
  PUTINI(INI::Section,INI::Entry,INI::String,INI::FileName)
  INI::Entry = 'XPosit'
  INI::String = TARGET{Prop:XPos}
  PUTINI(INI::Section,INI::Entry,INI::String,INI::FileName)
  INI::Entry = 'YPosit'
  INI::String = TARGET{Prop:YPos}
  PUTINI(INI::Section,INI::Entry,INI::String,INI::FileName)
  INI::Entry = 'Width'
  INI::String = TARGET{Prop:Width}
  PUTINI(INI::Section,INI::Entry,INI::String,INI::FileName)
  INI::Entry = 'Height'
  INI::String = TARGET{Prop:Height}
  PUTINI(INI::Section,INI::Entry,INI::String,INI::FileName)
  RETURN
